<?php

namespace Drupal\fieldcompose\Plugin\Field\FieldWidget;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Symfony\Component\Yaml\Yaml;

/**
 * Plugin implementation of the 'compose_default' widget.
 *
 * @FieldWidget(
 *   id = "compose_default",
 *   label = @Translation("Compose"),
 *   weight = 0,
 *   field_types = {
 *     "compose"
 *   }
 * )
 */
class ComposeWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $fieldsAnnotation = Yaml::parse($this->fieldDefinition->getSetting('fields') ?: '') ?? [];
    $item = $items[$delta];

    $fieldIds = array_filter(array_map(function ($fieldAnnotation) {
      return $fieldAnnotation['id'] ?? NULL;
    }, $fieldsAnnotation));

    if (!empty($item->value)) {
      $values = json_decode($item->value, JSON_OBJECT_AS_ARRAY);
    }

    foreach ($fieldIds as $key => $fieldId) {
      foreach ($fieldsAnnotation[$key] as $propName => $propVal) {
        if ($propName === 'id') {
          continue;
        }
        if ($propName[0] === '_') {
          $propName[0] = '#';
          $element[$fieldId][$propName] = $propVal;
        }
      }

      if (isset($values[$fieldId])) {
        $element[$fieldId]['#default_value'] = $values[$fieldId];
      }
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {

    foreach ($values as &$value) {
      if (isset($value['_original_delta'])) {
        unset($value['_original_delta']);
      }
      $value = json_encode($value, JSON_HEX_APOS | JSON_FORCE_OBJECT);
    }

    return $values;
  }

}
